@@ -136,7 +136,7 @@ static int server_transaction(ddns_t *ctx, ddns_info_t *provider)
 
 	client = &provider->checkip;
 	client->ssl_enabled = provider->checkip_ssl;
-	DO(http_init(client, "Checking for IP# change"));
+	DO(http_init(client, "Checking for IP# change", strstr(provider->system->name, "ipv6") ? TCP_FORCE_IPV6 : TCP_FORCE_IPV4));
 
 	/* Prepare request for IP server */
 	memset(ctx->work_buf, 0, ctx->work_buflen);
@@ -462,7 +462,7 @@ static int get_address_backend(ddns_t *ctx, ddns_info_t *info, char *address, si
 {
 	char name[sizeof(info->checkip_name.name)];
 	char url[sizeof(info->checkip_url)];
-	int rc;
+	int ssl, rc;
 
 	logit(LOG_DEBUG, "Get address for %s", info->system->name);
 	memset(address, 0, len);
@@ -470,7 +470,12 @@ static int get_address_backend(ddns_t *ctx, ddns_info_t *info, char *address, si
 	if (!get_address_cmd   (ctx, info, address, len))
 		return 0;
 
-	if (!get_address_iface (ctx, iface,address, len))
+	/* Check info specific interface */
+	if (!get_address_iface (ctx, info->ifname, address, len))
+		return 0;
+
+	/* Check the global interface */
+	if (!get_address_iface (ctx, iface, address, len))
 		return 0;
 
 	if (!get_address_remote(ctx, info, address, len))
@@ -484,24 +489,28 @@ static int get_address_backend(ddns_t *ctx, ddns_info_t *info, char *address, si
 	if (strstr(info->checkip_name.name, DDNS_MY_IP_SERVER))
 		goto error;
 
-	logit(LOG_WARNING, "Retrying with built-in 'default', api.ipify.org ...");
+	logit(LOG_WARNING, "Retrying with built-in 'default', http://ifconfig.me/ip ...");
 
 	/* keep backup, for now, future extension: count failures and replace permanently */
 	strlcpy(name, info->checkip_name.name, sizeof(name));
 	strlcpy(url, info->checkip_url, sizeof(url));
+	ssl = info->checkip_ssl;
 
-	/* Retry with http(s)://api.ipify.org/ */
+	/* Retry with http(s)://ifconfig.me/ip */
 	strlcpy(info->checkip_name.name, DDNS_MY_IP_SERVER, sizeof(info->checkip_name.name));
-	strlcpy(info->checkip_url, "/", sizeof(info->checkip_url));
+	strlcpy(info->checkip_url, DDNS_MY_CHECKIP_URL, sizeof(info->checkip_url));
+	info->checkip_ssl = DDNS_MY_IP_SSL;
 	rc = get_address_remote(ctx, info, address, len);
 
 	/* restore backup, for now, the official server may just have temporary problems */
 	strlcpy(info->checkip_name.name, name, sizeof(info->checkip_name.name));
 	strlcpy(info->checkip_url, url, sizeof(info->checkip_url));
+	info->checkip_ssl = ssl;
 
 	if (!rc) {
-		logit(LOG_WARNING, "Please note, %s seems unstable, consider overriding it "
-		      "in your configuration with 'checkip-server = default'", info->checkip_name.name);
+		logit(LOG_WARNING, "Please note, http%s://%s%s seems unstable, consider overriding it in"
+		      "your configuration with 'checkip-server = default'", info->checkip_ssl ? "s" : "",
+		      info->checkip_name.name, info->checkip_url);
 		return 0;
 	}
 
@@ -559,7 +568,7 @@ static int time_to_check(ddns_t *ctx, ddns_alias_t *alias)
 {
 	time_t past_time = time(NULL) - alias->last_update;
 
-	return ctx->force_addr_update ||
+	return alias->force_addr_update ||
 		(past_time > ctx->forced_update_period_sec);
 }
 
@@ -610,10 +619,10 @@ static int send_update(ddns_t *ctx, ddns_info_t *info, ddns_alias_t *alias, int
 		DO(info->system->setup(ctx, info, alias));
 
 	client->ssl_enabled = info->ssl_enabled;
-	rc = http_init(client, "Sending IP# update to DDNS server");
+	rc = http_init(client, "Sending IP# update to DDNS server", strstr(info->system->name, "ipv6") ? TCP_FORCE_IPV6 : TCP_FORCE_IPV4);
 	if (rc) {
 		/* Update failed, force update again in ctx->cmd_check_period seconds */
-		ctx->force_addr_update = 1;
+		alias->force_addr_update = 1;
 		return rc;
 	}
 
@@ -645,25 +654,25 @@ static int send_update(ddns_t *ctx, ddns_info_t *info, ddns_alias_t *alias, int
 		/* Update failed, force update again in ctx->cmd_check_period seconds */
 		logit(LOG_WARNING, "HTTP(S) Transaction failed, error %d: %s", rc, error_str(rc));
 		logit(LOG_INFO, "Update failed, forced update/retry in %d sec ...", ctx->cmd_check_period);
-		ctx->force_addr_update = 1;
+		alias->force_addr_update = 1;
 		goto exit;
 	}
 	logit(LOG_DEBUG, "DDNS server response: %s", trans.rsp);
 
 	rc = info->system->response(&trans, info, alias);
 	if (rc) {
-		logit(LOG_WARNING, "%s error in DDNS server response:",
-		      rc == RC_DDNS_RSP_RETRY_LATER ? "Temporary" : "Fatal");
-		logit(LOG_WARNING, "[%d %s] %s", trans.status, trans.status_desc,
-		      trans.rsp_body != trans.rsp ? trans.rsp_body : "");
+		logit(LOG_WARNING, "%s error in DDNS server response: %s",
+		      rc == RC_DDNS_RSP_RETRY_LATER || rc == RC_DDNS_RSP_TOO_FREQUENT ? "Temporary" : "Fatal", error_str(rc));
+//		logit(LOG_WARNING, "[%d %s]", trans.status, trans.status_desc);
+		logit(LOG_DEBUG, "%s", trans.rsp_body != trans.rsp ? trans.rsp_body : "");
 
 		/* Update failed, force update again in ctx->cmd_check_period seconds */
-		ctx->force_addr_update = 1;
+		alias->force_addr_update = 1;
 	} else {
 		logit(LOG_INFO, "Successful alias table update for %s => new IP# %s",
 		      alias->name, alias->address);
 
-		ctx->force_addr_update = 0;
+		alias->force_addr_update = 0;
 		if (changed)
 			(*changed)++;
 	}
@@ -681,7 +690,7 @@ static int update_alias_table(ddns_t *ctx)
 	ddns_info_t *info;
 
 	/* Issue #15: On external trig. force update to random addr. */
-	if (ctx->force_addr_update && ctx->forced_update_fake_addr) {
+	if (ctx->forced_update_fake_addr) {
 		/* If the DDNS server responds with an error, we ignore it here,
 		 * since this is just to fool the DDNS server to register a a
 		 * change, i.e., an active user. */
@@ -691,15 +700,17 @@ static int update_alias_table(ddns_t *ctx)
 
 			for (i = 0; i < info->alias_count; i++) {
 				ddns_alias_t *alias = &info->alias[i];
-				char backup[sizeof(alias->address)];
+				if (alias->force_addr_update) {
+					char backup[sizeof(alias->address)];
 
-				strlcpy(backup, alias->address, sizeof(backup));
+					strlcpy(backup, alias->address, sizeof(backup));
 
-				/* Picking random address in 203.0.113.0/24 ... */
-				snprintf(alias->address, sizeof(alias->address), "203.0.113.%d", (rand() + 1) % 255);
-				TRY(send_update(ctx, info, alias, NULL));
+					/* Picking random address in 203.0.113.0/24 ... */
+					snprintf(alias->address, sizeof(alias->address), "203.0.113.%d", (rand() + 1) % 255);
+					TRY(send_update(ctx, info, alias, NULL));
 
-				strlcpy(alias->address, backup, sizeof(alias->address));
+					strlcpy(alias->address, backup, sizeof(alias->address));
+				}
 			}
 
 			info = conf_info_iterator(0);
@@ -732,7 +743,7 @@ static int update_alias_table(ddns_t *ctx)
 				alias->last_update = time(NULL);
 
 				/* Update cache file for this entry */
-				write_cache_file(alias);
+				write_cache_file(alias, info->system->name);
 			}
 
 			/* Run command or script on successful update. */
@@ -743,7 +754,7 @@ static int update_alias_table(ddns_t *ctx)
 		if (RC_DDNS_RSP_NOTOK == rc || RC_DDNS_RSP_AUTH_FAIL == rc)
 			remember = rc;
 
-		if (RC_DDNS_RSP_RETRY_LATER == rc && !remember)
+		if ((RC_DDNS_RSP_RETRY_LATER == rc || rc == RC_DDNS_RSP_TOO_FREQUENT) && !remember)
 			remember = rc;
 
 		info = conf_info_iterator(0);
@@ -902,6 +913,7 @@ static int check_error(ddns_t *ctx, int rc)
 	case RC_OS_INVALID_IP_ADDRESS:
 	case RC_DDNS_RSP_RETRY_LATER:
 	case RC_DDNS_INVALID_CHECKIP_RSP:
+	case RC_DDNS_RSP_TOO_FREQUENT:
 		ctx->update_period = ctx->error_update_period_sec;
 		logit(LOG_WARNING, "Will retry again in %d sec ...", ctx->update_period);
 		break;
@@ -927,6 +939,7 @@ static int check_error(ddns_t *ctx, int rc)
 int ddns_main_loop(ddns_t *ctx)
 {
 	int rc = 0;
+	ddns_info_t *info;
 	static int first_startup = 1;
 
 	if (!ctx)
@@ -955,7 +968,15 @@ int ddns_main_loop(ddns_t *ctx)
 		}
 		if (ctx->cmd == CMD_FORCED_UPDATE) {
 			logit(LOG_INFO, "FORCED_UPDATE command received, updating now.");
-			ctx->force_addr_update = 1;
+			info = conf_info_iterator(1);
+			while (info) {
+				size_t i;
+				for (i = 0; i < info->alias_count; i++) {
+					ddns_alias_t *alias = &info->alias[i];
+					alias->force_addr_update = 1;
+				}
+				info = conf_info_iterator(0);
+			}
 			ctx->cmd = NO_CMD;
 		} else if (ctx->cmd == CMD_CHECK_NOW) {
 			logit(LOG_INFO, "CHECK_NOW command received, leaving startup delay.");
@@ -967,9 +988,17 @@ int ddns_main_loop(ddns_t *ctx)
 	DO(read_cache_file(ctx));
 	DO(get_encoded_user_passwd());
 
-	if (once && force)
-		ctx->force_addr_update = 1;
-
+	if (once && force) {
+			info = conf_info_iterator(1);
+			while (info) {
+				size_t i;
+				for (i = 0; i < info->alias_count; i++) {
+					ddns_alias_t *alias = &info->alias[i];
+					alias->force_addr_update = 1;
+				}
+				info = conf_info_iterator(0);
+			}
+	}
 	/* Initialization done, create pidfile to indicate we are ready to communicate */
 	if (once == 0 && pidfile_name[0] && pidfile(pidfile_name))
 		logit(LOG_WARNING, "Failed creating pidfile: %s", strerror(errno));
@@ -1009,7 +1038,16 @@ int ddns_main_loop(ddns_t *ctx)
 		}
 		if (ctx->cmd == CMD_FORCED_UPDATE) {
 			logit(LOG_INFO, "FORCED_UPDATE command received, updating now.");
-			ctx->force_addr_update = 1;
+
+			info = conf_info_iterator(1);
+			while (info) {
+				size_t i;
+				for (i = 0; i < info->alias_count; i++) {
+					ddns_alias_t *alias = &info->alias[i];
+					alias->force_addr_update = 1;
+				}
+				info = conf_info_iterator(0);
+			}
 			ctx->cmd = NO_CMD;
 			continue;
 		}
